package fr.atlantique.imt.inf211.jobmngt.controller;

import java.util.List;
import java.util.Optional;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import fr.atlantique.imt.inf211.jobmngt.dao.AppUserDao;
import fr.atlantique.imt.inf211.jobmngt.dao.CompanyDao;
import fr.atlantique.imt.inf211.jobmngt.entity.AppUser;
import fr.atlantique.imt.inf211.jobmngt.entity.Company;

@RestController
@RequestMapping("/companies")
public class CompanyController {

    @Autowired
    private CompanyDao companyDao;
    
    @Autowired
    private AppUserDao appUserDao;

    /**
     * Créer une entreprise
     * @param company L'entreprise envoyée en JSON
     * @return Message de confirmation ou d'erreur
     */
    @Transactional
    @PostMapping("/create")
    public ResponseEntity<String> createCompany(@RequestBody Company company) {
        try {
            // Vérification de l'utilisateur
            if (company.getAppUser() == null || company.getAppUser().getMail() == null) {
                return ResponseEntity.badRequest().body("Erreur : les informations de l'utilisateur sont invalides.");
            }

            // Vérifier si l'utilisateur existe déjà
            Optional<AppUser> existingUser = appUserDao.findByMail(company.getAppUser().getMail());
            if (existingUser.isPresent()) {
                return ResponseEntity.badRequest().body("Erreur : Un utilisateur avec cet email existe déjà.");
            }

            // Persist de l'utilisateur
            appUserDao.persist(company.getAppUser());

            // Hibernate devrait auto-générer l'ID, donc on le récupère après persistance
            company.setAppUser(company.getAppUser()); 

            // Vérification des champs obligatoires
            if (company.getName() == null || company.getName().isEmpty()) {
                return ResponseEntity.badRequest().body("Erreur : La dénomination de l'entreprise est obligatoire.");
            }

            // Persist de l'entreprise
            companyDao.persist(company);
            System.out.println("✅ Entreprise ajoutée : " + company.getName());

            return ResponseEntity.ok("Entreprise créée avec succès.");
        } catch (Exception e) {
            System.err.println("❌ Erreur lors de la création de l'entreprise : " + e.getMessage());
            return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR)
                    .body("Erreur lors de la création de l'entreprise : " + e.getMessage());
        }
    }

    /**
     * Supprimer une entreprise par ID
     */
    @Transactional
    @DeleteMapping("/delete/{id}")
    public ResponseEntity<String> deleteCompany(@PathVariable int id) {
        Company c = companyDao.findById(id);
        if (c != null) {
            companyDao.remove(c);
            System.out.println("🗑️ Entreprise supprimée : " + c.getName());
            return ResponseEntity.ok("Entreprise supprimée avec succès.");
        }
        return ResponseEntity.status(HttpStatus.NOT_FOUND).body("❌ Erreur : Entreprise non trouvée.");
    }

    /**
     * Récupérer toutes les entreprises
     */
    @Transactional(readOnly = true)
    @GetMapping("/list")
    public List<Company> listCompanies() {
        List<Company> companies = companyDao.findAll("id", "asc");
        System.out.println("📌 Nombre d'entreprises trouvées : " + companies.size());
        return companies;
    }

    /**
     * Trouver une entreprise par ID
     */
    @Transactional(readOnly = true)
    @GetMapping("/find/{id}")
    public ResponseEntity<Company> getCompanyById(@PathVariable int id) {
        Company company = companyDao.findById(id);
        if (company != null) {
            System.out.println("📌 Entreprise trouvée : " + company.getName());
            return ResponseEntity.ok(company);
        } else {
            return ResponseEntity.status(HttpStatus.NOT_FOUND).build();
        }
    }
}
